﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Moosta.Common.Song;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using Moosta.Common.Analyze;
using Moosta.Common;

namespace OMPPlugins
{
    public class FFMpegAudioDecoder : IAudioDecoder
    {
        #region IAudioDecoder メンバ

        private string[] supportedExtensions = { ".mp3", ".wav", ".aac", ".mp4", ".m4a", ".3gp" };

        string ffmpegPath = Path.Combine(Application.StartupPath, "plugins\\ffmpeg.exe");

        string tmpPlayFileName = Path.GetTempFileName() + ".wav";

        public void SetApplicationVersion(string versionStr)
        {
            tmpPlayFileName = Path.Combine(Path.GetTempPath(),"charaminTmpWave" + versionStr.Replace(".","") + ".wav");
        }

        public string[] GetSupportedExtensions()
        {
            return supportedExtensions;
        }

        public FFMpegAudioDecoder()
        {
            string extPath = Path.Combine(Application.StartupPath , "plugins\\OMPPlugins_extensions.txt");
            if (File.Exists(extPath))
            {
                List<string> exts = new List<string>();
                using (StreamReader sr = new StreamReader(extPath))
                {
                    while (sr.Peek() > -1)
                    {
                        try
                        {
                            string line = sr.ReadLine();
                            if (string.IsNullOrEmpty(line)) continue;

                            exts.Add(line);
                        }
                        catch
                        {

                        }
                    }
                }

                supportedExtensions = exts.ToArray();
            }
        }
        

        public byte[] Decode(string fileName, int channel, int smaplerate, int samplebits)
        {
           

            string outFileName = Path.GetTempFileName() + ".wav";

            string command = string.Format(
                "-y -i \"{0}\" -vn -acodec {1} -ac {2} -ar {3} -f wav \"{4}\"",
                fileName, 
                (samplebits == 16) ? "pcm_s16le" : "pcm_u8",
                channel,
                smaplerate,
                outFileName);

            ProcessStartInfo psInfo = new ProcessStartInfo();

            psInfo.FileName = ffmpegPath;
            psInfo.CreateNoWindow = true;
            psInfo.UseShellExecute = false;
            psInfo.Arguments = command;
            psInfo.RedirectStandardOutput = true;

            System.Diagnostics.Process hProcess = Process.Start(psInfo);

            // 終了するまで待機する
            hProcess.WaitForExit();

            string output = hProcess.StandardOutput.ReadToEnd();



            byte[] ww = null;

            //Wavファイルを読み込み
            WaveFileFormat format = new WaveFileFormat();

            if (File.Exists(outFileName))
            {
                using (FileStream fs = new FileStream(outFileName, FileMode.Open, FileAccess.Read))
                {
                    using (BinaryReader br = new BinaryReader(fs))
                    {
                        ww = WaveFileManager.ReadWaveFile(br, out format);
                    }
                }

                File.Delete(outFileName);
            }

            hProcess.Close();
            hProcess.Dispose();
            return ww;
        }

        public string DecoderName
        {
            get { return "FFMpeg Audio Decoder"; }
        }



        public string GetPlayableAudioFileName(string fileName)
        {

            string playFileName = fileName;
            string ext = Path.GetExtension(fileName).ToLower();

            //MP3ファイルは、タグを消去しないと再生できないことがある
            if (ext.Equals(".mp3"))
            {
                playFileName = Mp3AudioDecoder.GetPlayableMP3FileName(fileName);
            }
            else if (ext.Equals(".wav"))
            {
                playFileName = fileName;
            }
            else
            {
            	//DirectShowが標準では対応していないため、
            	//その他のファイルは一度Waveファイルにデコードする。
                string command = string.Format(
                    "-y -i \"{0}\" -vn -acodec pcm_s16le -ac 2 -ar 44100 -f wav \"{1}\"",
                    fileName, tmpPlayFileName);

                ProcessStartInfo psInfo = new ProcessStartInfo();

                psInfo.FileName = ffmpegPath;
                psInfo.CreateNoWindow = true;
                psInfo.UseShellExecute = false;
                psInfo.Arguments = command;
                psInfo.RedirectStandardOutput = true;

                System.Diagnostics.Process hProcess = Process.Start(psInfo);

                hProcess.WaitForExit();

                string output = hProcess.StandardOutput.ReadToEnd();

                hProcess.Close();
                hProcess.Dispose();

                playFileName = tmpPlayFileName;
            }

            return playFileName;
        }


        public AudioFileInformation LoadInformation(string fileName)
        {
            AudioFileInformation audioInfo = new AudioFileInformation();
            audioInfo.Title = Path.GetFileNameWithoutExtension(fileName);

            string ext = Path.GetExtension(fileName).ToLower();

            //MP3ファイルはタグを読み込む
            if (ext.Equals(".mp3"))
            {
                try
                {

                    ID3Tag tag = ID3Tag.FromMP3File(fileName);
                    if (!string.IsNullOrEmpty(tag.Title))
                    {
                        audioInfo.Title = tag.Title;
                    }

                    audioInfo.Album = tag.Album;
                    audioInfo.Artist = tag.Artist;

                }
                catch
                {
                   


                }
            }

            return audioInfo;
        }

        public void ReleaseResources()
        {
            Mp3AudioDecoder.DeleteTmpMp3File();

            try
            {
                if (File.Exists(tmpPlayFileName))
                {
                    File.Delete(tmpPlayFileName);
                }
            }
            catch
            {
            	
            }
        }


        /// <summary>
        /// 設定ダイアログを表示します。
        /// </summary>
        /// <param name="parent"></param>
        public void ShowSettingDialog(IWin32Window parent)
        {
        }

        /// <summary>
        /// 設定ダイアログをもつかどうかを取得します。
        /// </summary>
        /// <returns></returns>
        public bool IsHaveSettingDialog { get { return false; } }

        #endregion
    }
}
